import hof_c, hof_py, time, unittest

class IteratorAdaptee:
    
    def __init__(self, seq):
        self.seq = seq
        
    def __len__(self):
        return len(self.seq)

    def __iter__(self):
        return iter(self.seq)

    
class TestHigherOrderFunctions(object):

    seq = [x for x in xrange(10)]

    def iteradaptee(self, seq):
        return IteratorAdaptee(seq)

    def _testiteradaptee(self, f, seq, expr):
        try:
            res = f(self.iteradaptee(seq))
        except:
            self.fail('fail iterator protocol')
        self.assertEqual(res, expr)

    def test_find_empty(self):
        f = lambda: self.hof.find(lambda x: True, [])
        self.assertRaises(LookupError, f)

    def test_find_success(self):
        f = lambda x: x == 3
        self.assertEqual(self.hof.find(f, [0,1,2,3,4,5]), 3)

    def test_find_fail(self):
        f = lambda x: x == 3
        test = lambda: self.hof.find(f, [0,1,2,4,5])
        self.assertRaises(LookupError, test)

    def test_find_iteradaptee(self):
        f = lambda seq: self.hof.find(lambda x: True, seq)
        self._testiteradaptee(f, [0,1,2], 0)

    def test_find_index_empty(self):
        f = lambda: self.hof.find_index(lambda x: True, [])
        self.assertRaises(LookupError, f)

    def test_find_index_success(self):
        f = lambda x: x == 3
        self.assertEqual(self.hof.find_index(f, [0,1,2,3,4,5]), 3)

    def test_find_index_fail(self):
        f = lambda x: x == 3
        test = lambda: self.hof.find_index(f, [0,1,2,4,5])
        self.assertRaises(LookupError, test)

    def test_find_index_iteradaptee(self):
        f = lambda seq: self.hof.find_index(lambda x: True, seq)
        self._testiteradaptee(f, [0,1,2], 0)

    def test_find_indices_empty(self):
        res = self.hof.find_indices(lambda x: True, [])
        self.assertEqual(res, [])

    def test_find_indices_success(self):
        f = lambda x: x == 3
        self.assertEqual(self.hof.find_indices(f, [0,1,2,3,4,5]), [3])

    def test_find_indices_fail(self):
        f = lambda x: x == 3
        res = self.hof.find_indices(f, [0,1,2,4,5])
        self.assertEqual(res, [])

    def test_find_indices_iteradaptee(self):
        f = lambda seq: self.hof.find_indices(lambda x: True, seq)
        self._testiteradaptee(f, [0,1,2], [0,1,2])

    def test_all_empty(self):
        res = self.hof.all(lambda x: True, [])
        self.assertEqual(res, False)

    def test_all_success(self):
        f = lambda x: x > -1
        self.assertEqual(self.hof.all(f, [0,1,2,3,4,5]), True)

    def test_all_fail(self):
        f = lambda x: x > 0
        res = self.hof.all(f, [0,1,2,3,4,5])
        self.assertEqual(res, False)

    def test_all_iteradaptee(self):
        f = lambda seq: self.hof.all(lambda x: True, seq)
        self._testiteradaptee(f, [0,1,2], True)
        
    def test_any_empty(self):
        res = self.hof.any(lambda x: True, [])
        self.assertEqual(res, False)

    def test_any_success(self):
        f = lambda x: x == 2
        self.assertEqual(self.hof.any(f, [0,1,2,3,4,5]), True)

    def test_any_fail(self):
        f = lambda x: x < 0
        res = self.hof.any(f, [0,1,2,3,4,5])
        self.assertEqual(res, False)

    def test_any_iteradaptee(self):
        f = lambda seq: self.hof.any(lambda x: x == 2, seq)
        self._testiteradaptee(f, [0,1,2], True)

    def test_span_empty(self):
        res = self.hof.span(lambda x: True, [])
        self.assertEqual(res, ([], []))

    def test_span_success(self):
        f = lambda x: x < 3
        self.assertEqual(self.hof.span(f, [0,1,2,3,4,5]),
                         ([0,1,2], [3,4,5]))

    def test_span_fail(self):
        f = lambda x: x < 0
        res = self.hof.span(f, [0,1,2,3,4,5])
        self.assertEqual(res, ([], [0,1,2,3,4,5]))

    def test_span_iteradaptee(self):
        f = lambda seq: self.hof.span(lambda x: x < 2, seq)
        self._testiteradaptee(f, [0,1,2], ([0,1],[2]))

    def test_take_empty(self):
        res = self.hof.take(lambda x: True, [])
        self.assertEqual(res, [])

    def test_take_success(self):
        f = lambda x: x < 3
        self.assertEqual(self.hof.take(f, [0,1,2,3,4,5]), [0,1,2])

    def test_take_fail(self):
        f = lambda x: x < 0
        res = self.hof.take(f, [0,1,2,3,4,5])
        self.assertEqual(res, [])

    def test_take_iteradaptee(self):
        f = lambda seq: self.hof.take(lambda x: x < 2, seq)
        self._testiteradaptee(f, [0,1,2], [0,1])

    def test_drop_empty(self):
        res = self.hof.drop(lambda x: True, [])
        self.assertEqual(res, [])

    def test_drop_success(self):
        f = lambda x: x < 3
        self.assertEqual(self.hof.drop(f, [0,1,2,3,4,5]), [3,4,5])

    def test_drop_fail(self):
        f = lambda x: x > -1
        res = self.hof.drop(f, [0,1,2,3,4,5])
        self.assertEqual(res, [])

    def test_drop_iteradaptee(self):
        f = lambda seq: self.hof.drop(lambda x: x < 2, seq)
        self._testiteradaptee(f, [0,1,2], [2])

    def test_partition_empty(self):
        res = self.hof.partition(lambda x: True, [])
        self.assertEqual(res, ([], []))

    def test_partition_both(self):
        f = lambda x: x % 2 == 0
        self.assertEqual(self.hof.partition(f, [0,1,2,3,4,5]),
                         ([0,2,4],[1,3,5]))

    def test_partition_true(self):
        f = lambda x: True
        self.assertEqual(self.hof.partition(f, [0,1,2,3,4,5]),
                         ([0,1,2,3,4,5], []))

    def test_partition_false(self):
        f = lambda x: False
        self.assertEqual(self.hof.partition(f, [0,1,2,3,4,5]),
                         ([], [0,1,2,3,4,5]))

    def test_partition_iteradaptee(self):
        f = lambda seq: self.hof.partition(lambda x: x == 2, seq)
        self._testiteradaptee(f, [0,1,2], ([2], [0,1]))


class TestHigherOrderFunctionsScript(unittest.TestCase,
                                     TestHigherOrderFunctions):

    hof = hof_py


class TestHigherOrderFunctionsExtension(unittest.TestCase,
                                        TestHigherOrderFunctions):

    hof = hof_c


class TestScale(unittest.TestCase):
#class TestScale(object):
    
    def test_scale(self):
        print
        print "=== test scale ==="
    
        n = 5000
        seq = [x for x in xrange(1000)]
        
        f = lambda x: x == 999
        self.bench_both(n, 'find_indices', f, seq)
        self.bench_both(n, 'find_index', f, seq)
        self.bench_both(n, 'find', f, seq)
        
        f = lambda x: x == 999
        self.bench_both(n, 'any', f, seq)
        
        f = lambda x: x < 2000
        self.bench_both(n, 'all', f, seq)
        
        f = lambda x: x < 998
        self.bench_both(n, 'take', f, seq)
        self.bench_both(n, 'drop', f, seq)
        self.bench_both(n, 'span', f, seq)
        
        f = lambda x: True if x % 2 == 0 else False
        self.bench_both(n, 'partition', f, seq)

    def bench_both(self, n, attr, f, *args):
        print "%s:" % attr
        pytime = self.bench(n, getattr(hof_py, attr), f, *args)
        print "    python    = %f" % pytime
        ctime = self.bench(n, getattr(hof_c, attr), f, *args)
        print "    extension = %f (%2d%% faster)" % \
              (ctime, (1 - ctime / pytime) * 100)

    def bench(self, n, f, *args):
        start = time.time()
        for i in xrange(0, n):
            f(*args)
        return time.time() - start


if __name__ == '__main__':
    unittest.main()
